@@ -1,4 +1,4 @@
-__version__ = "0.4.0"
+__version__ = "0.5.6"
 from .bins import *
 from .perf import *
 from .stats import *
@@ -8,3 +8,5 @@ from .logit import *
 from .regression import *
 from .basics import *
 from .visualize import *
+from .notebook import *
+from .example_data import *
