@@ -1,3 +1,4 @@
+# from turtle import title
 import pandas as pd
 import numpy as np
 import matplotlib.pyplot as plt
@@ -5,6 +6,7 @@ import seaborn as sns
 from pyrsm import xtile
 from pyrsm.utils import ifelse
 from sklearn import metrics
+from scipy.stats import rankdata
 
 
 def calc_qnt(df, rvar, lev, pred, qnt=10):
@@ -32,32 +34,18 @@ def calc_qnt(df, rvar, lev, pred, qnt=10):
     df = df.loc[:, (rvar, pred)]
     df["bins"] = xtile(df[pred], qnt)
     df["rvar_int"] = np.where(df[rvar] == lev, 1, 0)
-    perf_df = (
-        df.groupby("bins")["rvar_int"].agg(nr_obs="count", nr_resp=sum).reset_index()
-    )
+    perf_df = df.groupby("bins").rvar_int.agg(nr_obs="count", nr_resp=sum).reset_index()
 
     # flip if needed
-    if perf_df["nr_resp"].iloc[1] < perf_df["nr_resp"].iloc[-1]:
+    if perf_df.nr_resp.iloc[1] < perf_df.nr_resp.iloc[-1]:
         perf_df = perf_df.sort_values("bins", ascending=False)
 
-    perf_df["cum_obs"] = np.cumsum(perf_df["nr_obs"])
-    perf_df["cum_prop"] = perf_df["cum_obs"] / perf_df["cum_obs"].iloc[-1]
-    perf_df["cum_resp"] = np.cumsum(perf_df["nr_resp"])
+    perf_df["cum_obs"] = np.cumsum(perf_df.nr_obs)
+    perf_df["cum_prop"] = perf_df.cum_obs / perf_df.cum_obs.iloc[-1]
+    perf_df["cum_resp"] = np.cumsum(perf_df.nr_resp)
     return perf_df
 
 
-def calc_dec(df, rvar, lev, pred, qnt=10):
-    """Deprecated function. Use calc_qnt instead"""
-    print("The 'calc_dec' function is deprecated. Use 'calc_qnt' instead")
-    return calc_dec(df, rvar, lev, pred, qnt=10)
-
-
-def calc(df, rvar, lev, pred, qnt=10):
-    """Deprecated function. Use calc_qnt instead"""
-    print("The 'calc' function is deprecated. Use 'calc_qnt' instead")
-    return None
-
-
 def gains_tab(df, rvar, lev, pred, qnt=10):
     """
     Calculate cumulative gains using the cum_resp column
@@ -81,9 +69,9 @@ def gains_tab(df, rvar, lev, pred, qnt=10):
     """
 
     df = calc_qnt(df, rvar, lev, pred, qnt=qnt)
-    df["cum_gains"] = df["cum_resp"] / df["cum_resp"].iloc[-1]
+    df["cum_gains"] = df.cum_resp / df.cum_resp.iloc[-1]
     df0 = pd.DataFrame({"cum_prop": [0], "cum_gains": [0]})
-    df = pd.concat([df0, df], sort=False)
+    df = pd.concat([df0, df], sort=False).reset_index(drop=True)
     df.index = range(df.shape[0])
     return df[["cum_prop", "cum_gains"]]
 
@@ -111,8 +99,8 @@ def lift_tab(df, rvar, lev, pred, qnt=10):
     """
 
     df = calc_qnt(df, rvar, lev, pred, qnt=qnt)
-    df["cum_resp_rate"] = df["cum_resp"] / df["cum_obs"]
-    df["cum_lift"] = df["cum_resp_rate"] / df["cum_resp_rate"].iloc[-1]
+    df["cum_resp_rate"] = df.cum_resp / df.cum_obs
+    df["cum_lift"] = df.cum_resp_rate / df.cum_resp_rate.iloc[-1]
     df.index = range(df.shape[0])
     return df[["cum_prop", "cum_lift"]]
 
@@ -153,9 +141,9 @@ def confusion(df, rvar, lev, pred, cost=1, margin=2):
     gtbe = df[pred] > break_even
     pos = df[rvar] == lev
     TP = np.where(gtbe & pos, 1, 0).sum()
-    FP = np.where((gtbe == True) & (pos == False), 1, 0).sum()
-    TN = np.where((gtbe == False) & (pos == False), 1, 0).sum()
-    FN = np.where((gtbe == False) & (pos == True), 1, 0).sum()
+    FP = np.where(gtbe & np.logical_not(pos), 1, 0).sum()
+    TN = np.where(np.logical_not(gtbe) & np.logical_not(pos), 1, 0).sum()
+    FN = np.where(np.logical_not(gtbe) & pos, 1, 0).sum()
     contact = (TP + FP) / (TP + FP + TN + FN)
     return TP, FP, TN, FN, contact
 
@@ -306,7 +294,7 @@ def profit_tab(df, rvar, lev, pred, qnt=10, cost=1, margin=2):
     """
 
     df = calc_qnt(df, rvar, lev, pred, qnt=qnt)
-    df["cum_profit"] = margin * df["cum_resp"] - cost * df["cum_obs"]
+    df["cum_profit"] = margin * df.cum_resp - cost * df.cum_obs
     df0 = pd.DataFrame({"cum_prop": [0], "cum_profit": [0]})
     df = pd.concat([df0, df], sort=False)
     df.index = range(df.shape[0])
@@ -340,9 +328,9 @@ def ROME_tab(df, rvar, lev, pred, qnt=10, cost=1, margin=2):
     """
 
     df = calc_qnt(df, rvar, lev, pred, qnt=qnt)
-    df["cum_profit"] = margin * df["cum_resp"] - cost * df["cum_obs"]
-    cum_cost = cost * df["cum_obs"]
-    df["ROME"] = (margin * df["cum_resp"] - cum_cost) / cum_cost
+    df["cum_profit"] = margin * df.cum_resp - cost * df.cum_obs
+    cum_cost = cost * df.cum_obs
+    df["ROME"] = (margin * df.cum_resp - cum_cost) / cum_cost
     df.index = range(df.shape[0])
     return df[["cum_prop", "ROME"]]
 
@@ -398,10 +386,10 @@ def profit_plot(
         for k in dct.keys()
         for p in pred
     ]
-    df = pd.concat(df)
+    df = pd.concat(df).reset_index(drop=True)
     fig = sns.lineplot(
         x="cum_prop", y="cum_profit", data=df, hue=group, marker=marker, **kwargs
-    )
+    ).legend(title=None)
     fig.set(ylabel="Profit", xlabel="Proportion of customers")
     fig.axhline(1, linestyle="--", linewidth=1)
     if contact:
@@ -462,10 +450,10 @@ def ROME_plot(df, rvar, lev, pred, qnt=10, cost=1, margin=2, marker="o", **kwarg
         for k in dct.keys()
         for p in pred
     ]
-    rd = pd.concat(rd)
+    rd = pd.concat(rd).reset_index(drop=True)
     fig = sns.lineplot(
         x="cum_prop", y="ROME", data=rd, hue=group, marker=marker, **kwargs
-    )
+    ).legend(title=None)
     fig.set(
         ylabel="Return on Marketing Expenditures (ROME)",
         xlabel="Proportion of customers",
@@ -519,10 +507,10 @@ def gains_plot(df, rvar, lev, pred, qnt=10, marker="o", **kwargs):
         for k in dct.keys()
         for p in pred
     ]
-    rd = pd.concat(rd)
+    rd = pd.concat(rd).reset_index(drop=True)
     fig = sns.lineplot(
         x="cum_prop", y="cum_gains", data=rd, hue=group, marker=marker, **kwargs
-    )
+    ).legend(title=None)
     fig.set(ylabel="Cumulative gains", xlabel="Proportion of customers")
     plt.plot([0, 1], [0, 1], linestyle="--", linewidth=1)
     return fig
@@ -573,10 +561,10 @@ def lift_plot(df, rvar, lev, pred, qnt=10, marker="o", **kwargs):
         for k in dct.keys()
         for p in pred
     ]
-    rd = pd.concat(rd)
+    rd = pd.concat(rd).reset_index(drop=True)
     fig = sns.lineplot(
         x="cum_prop", y="cum_lift", data=rd, hue=group, marker=marker, **kwargs
-    )
+    ).legend(title=None)
     fig.axhline(1, linestyle="--", linewidth=1)
     fig.set(ylabel="Cumulative lift", xlabel="Proportion of customers")
     return fig
@@ -651,5 +639,41 @@ def evalbin(df, rvar, lev, pred, cost=1, margin=2, dec=3):
             result = result.append(calculate_metrics(key, val, p))
 
     result.index = range(result.shape[0])
-    result["index"] = result.groupby("Type")["profit"].transform(lambda x: x / x.max())
+    result["index"] = result.groupby("Type").profit.transform(lambda x: x / x.max())
     return result.round(dec)
+
+
+def auc(rvar, pred, lev=1):
+    """
+    Calculate area under the RO curve (AUC)
+
+    Calculation adapted from https://stackoverflow.com/a/50202118/1974918
+
+    Parameters
+    ----------
+    rvar : Pandas series or numpy vector
+        Vector with the response variable
+    pred : Pandas series or numpy vector
+        Vector with model predictions
+    lev : str
+        Name of the 'success' level in rvar
+
+    Returns
+    -------
+    float :
+        AUC metric
+
+    Examples
+    --------
+    auc(dvd.buy, np.random.uniform(size=20000), "yes")
+    auc(dvd.buy, rsm.ifelse(dvd.buy == "yes", 1, 0), "yes")
+    """
+    if type(rvar[0]) != bool or lev is not None:
+        rvar = rvar == lev
+
+    n1 = np.sum(np.logical_not(rvar))
+    n2 = np.sum(rvar)
+
+    U = np.sum(rankdata(pred)[np.logical_not(rvar)]) - n1 * (n1 + 1) / 2
+    wt = U / n1 / n2
+    return ifelse(wt < 0.5, 1 - wt, wt)
