@@ -3,6 +3,8 @@ import pandas as pd
 from itertools import product
 from datetime import date, datetime
 from math import ceil
+from IPython.display import display, Markdown
+from sys import modules
 
 
 def add_description(df, md="", path=""):
@@ -35,10 +37,18 @@ def add_description(df, md="", path=""):
     return df
 
 
-def describe(df):
-    """Print out Pandas dataframe description attribute if available. Else use Pandas' description method to provide summary statistics"""
+def describe(df, prn=True):
+    """
+    Print out Pandas dataframe description attribute if available. Else use Pandas'
+    description method to provide summary statistics
+    """
     if hasattr(df, "description"):
-        print(df.description)
+        if "ipykernel" in modules and prn:
+            display(Markdown(df.description))
+        elif prn:
+            print(df.description)
+        else:
+            return df.description
     else:
         print("No description attribute available")
         return df.describe()
@@ -135,15 +145,16 @@ def levels_list(df):
     return {str(col): list(df[col].unique()) for col in df.columns}
 
 
-def expand_grid(dct):
+def expand_grid(dct, dtypes=None):
     """
     Provide a dictionary and get a pandas dataframe back with all possible
     value combinations
 
     Parameters
     ----------
-    dct
-        Dictionary with value combinations to expand
+    dct : Dictionary with value combinations to expand
+    dtypes : Pandas series
+        Pandas column types extracted from a dataframe using df.dtypes
 
     Returns
     -------
@@ -153,7 +164,11 @@ def expand_grid(dct):
     -------
     expand_grid({"var1": ["a", "b"], "var2": [1, 2]})
     """
-    return pd.DataFrame([val for val in product(*dct.values())], columns=dct.keys())
+    df = pd.DataFrame([val for val in product(*dct.values())], columns=dct.keys())
+    if dtypes is not None:
+        return df.astype(dtypes)
+    else:
+        return df
 
 
 def table2data(df, freq):
